/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package dk.eclipse.sablecceditor;

import java.io.PushbackReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.BadPositionCategoryException;
import org.eclipse.jface.text.DefaultPositionUpdater;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IPositionUpdater;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;
import org.sablecc.sablecc2x.analysis.DepthFirstAdapter;
import org.sablecc.sablecc2x.lexer.Lexer;
import org.sablecc.sablecc2x.node.AHelperDef;
import org.sablecc.sablecc2x.node.AHelpers;
import org.sablecc.sablecc2x.node.AIdList;
import org.sablecc.sablecc2x.node.AIdListTail;
import org.sablecc.sablecc2x.node.AIgnTokens;
import org.sablecc.sablecc2x.node.AProd;
import org.sablecc.sablecc2x.node.AProductions;
import org.sablecc.sablecc2x.node.ATokenDef;
import org.sablecc.sablecc2x.node.ATokens;
import org.sablecc.sablecc2x.node.Start;
import org.sablecc.sablecc2x.node.TId;
import org.sablecc.sablecc2x.node.Token;
import org.sablecc.sablecc2x.parser.Parser;
		

	

	
	
	class ContentProvider implements ITreeContentProvider {
		private final SableccContentOutlinePage page;
		/**
		 * @param SableccContentOutlinePage
		 */
		ContentProvider(SableccContentOutlinePage page) {
			this.page = page;
			
		}
		/*
		 * fields modified and added:
		 * 1. segment string for 
		 * 			helper definition
		 * 			tokens
		 * 			ignored tokens
		 * 			productions
		 * 2. fContent is a list of lists.
		 */
		
		public final static String SEGMENTS = "__sablecc_segments"; //$NON-NLS-1$		protected final static String HELPERS = "__sablecc_HELPERS";
		public final static String TOKENS = "__sablecc_TOKENS";
		public final static String IGNRED_TOKENS =	"__sablecc_IGNORED_TOKENS";
		public final static String PRODUCTIONS = "__sablecc_PRODUCTIONS";
		public final static String HELPERS= "__sablecc_HELPERS";
		public class SableccCodePositioner extends DepthFirstAdapter{
			private IDocument doc;
			public SableccCodePositioner(IDocument document){
				doc=document;
				
			}
			
			public Position getDefaultPosition(Token tok){
				int ret;
				try {
					ret= tok.getPos()+ doc.getLineOffset(tok.getLine()-1);
					return new Position(ret);
				} catch (BadLocationException e) {
					
					e.printStackTrace();
				}
				return null;
			}
						
			/* (non-Javadoc)
			 * @see org.sablecc.sablecc2x.analysis.DepthFirstAdapter#inAHelpers(org.sablecc.sablecc2x.node.AHelpers)
			 */
			public void outAHelpers(AHelpers node) {
				// there is a helper section, so add it to fContent.
				// here we don't do semantic check, so is for other tokens.
				Segment helperSegment= new Segment(HELPERS, 
					getDefaultPosition(node.getHelpers()));
				getContent().add(helperSegment);
				
				List helperDefs= node.getHelperDefs();
				for (int i=0; i< helperDefs.size(); i++){
					AHelperDef aHelperDef= (AHelperDef)helperDefs.get(i);
					Segment aHelperDefSeg= new Segment(aHelperDef.getId().getText(),
									getDefaultPosition(aHelperDef.getId()));
					helperSegment.addChild(aHelperDefSeg);
					
				}
			}
		
	
			/* (non-Javadoc)
			 * @see org.sablecc.sablecc2x.analysis.DepthFirstAdapter#outATokens(org.sablecc.sablecc2x.node.ATokens)
			 */
			public void outATokens(ATokens node) {
				Segment tokenSegment= new Segment(TOKENS, 
					getDefaultPosition(node.getTokens()));
				getContent().add(tokenSegment);
				
				List tokenDefs= node.getTokenDefs();
				for (int i=0; i< tokenDefs.size(); i++){
					ATokenDef aTokenDef= (ATokenDef)tokenDefs.get(i);
					Segment aTokenDefSeg= new Segment(aTokenDef.getId().getText(),
									getDefaultPosition(aTokenDef.getId()));
					tokenSegment.addChild(aTokenDefSeg); 
				}
			}
	
			/* (non-Javadoc)
			 * @see org.sablecc.sablecc2x.analysis.DepthFirstAdapter#outAIgnTokens(org.sablecc.sablecc2x.node.AIgnTokens)
			 */
			public void outAIgnTokens(AIgnTokens node) {
				Segment igtokenSegment= new Segment(IGNRED_TOKENS, 
					getDefaultPosition(node.getIgnored()));
				getContent().add(igtokenSegment);
				AIdList igTokens= (AIdList)node.getIdList();
				if (igTokens!= null){
					// get first id.
					TId aIgnredId= igTokens.getId();
					Segment aTokenDefSeg= new Segment(aIgnredId.getText(),
									getDefaultPosition(aIgnredId));
					igtokenSegment.addChild(aTokenDefSeg);
					
					for (int i=0; i< igTokens.getIds().size();i++){
						AIdListTail aTToken= (AIdListTail)igTokens.getIds().get(i);
						Segment aTTokenSegment= new Segment(aTToken.getId().getText(),
									getDefaultPosition(aTToken.getId()));
						igtokenSegment.addChild(aTTokenSegment); 
					}
				}
			}
			
			/* (non-Javadoc)
			 * @see org.sablecc.sablecc2x.analysis.DepthFirstAdapter#outAProductions(org.sablecc.sablecc2x.node.AProductions)
			 */
			public void outAProductions(AProductions node) {
				Segment prodSegment= new Segment(PRODUCTIONS, 
					getDefaultPosition(node.getProductions()));
				getContent().add(prodSegment);
				
				List prodList= node.getProds();
				
				for (int i=0; i<prodList.size(); i++) {
					AProd aProdDef= (AProd) prodList.get(i);
					Segment aProdDefSeg= new Segment(aProdDef.getId().getText(),
							getDefaultPosition(aProdDef.getId()));
					prodSegment.addChild(aProdDefSeg); 
				}				
			}
	
	
		} //class
		
		
	
	
		protected IPositionUpdater fPositionUpdater =
			new DefaultPositionUpdater(SEGMENTS);
		private List fContent = new ArrayList();
	
		protected void parse(IDocument document) {
			try {
				Lexer lexer = new Lexer(
					new PushbackReader(
					new StringReader(document.get()),
							1024));
				Parser parser = new Parser(lexer);
				Start ast = parser.parse();
				//System.out.println(ast);
				ast.apply(new SableccCodePositioner(document));
			} catch (Exception e) {
				e.printStackTrace();
				//System.exit(1);
			}
	
		}
	
		/*
		 * @see IContentProvider#inputChanged(Viewer, Object, Object)
		 */
		public void inputChanged(
			Viewer viewer,
			Object oldInput,
			Object newInput) {
			if (oldInput != null) {
				IDocument document = this.page.fDocumentProvider.getDocument(oldInput);
				if (document != null) {
					try {
						document.removePositionCategory(SEGMENTS);
						document.removePositionCategory(HELPERS);
						document.removePositionCategory(TOKENS);
						document.removePositionCategory(IGNRED_TOKENS);
						document.removePositionCategory(PRODUCTIONS);
					} catch (BadPositionCategoryException x) {
					}
					document.removePositionUpdater(fPositionUpdater);
				}
			}
	
			fContent.clear();
	
			if (newInput != null) {
				IDocument document = this.page.fDocumentProvider.getDocument(newInput);
				if (document != null) {
					document.addPositionCategory(SEGMENTS);
					document.addPositionCategory(HELPERS);
					document.addPositionCategory(TOKENS);
					document.addPositionCategory(IGNRED_TOKENS);
					document.addPositionCategory(PRODUCTIONS);
					document.addPositionUpdater(fPositionUpdater);
					parse(document);
				}
			}
		}
	
		/*
		 * @see IContentProvider#dispose
		 */
		public void dispose() {
			if (fContent != null) {
				fContent.clear();
				fContent = null;
			}
		}
	
		/*
		 * @see IContentProvider#isDeleted(Object)
		 */
		public boolean isDeleted(Object element) {
			return false;
		}
	
		/*
		 * @see IStructuredContentProvider#getElements(Object)
		 */
		public Object[] getElements(Object element) {
			return fContent.toArray();
		}
	
		/*
		 * @see ITreeContentProvider#hasChildren(Object)
		 */
		public boolean hasChildren(Object element) {
			if (!(element instanceof Segment))
			return false;
			
			return ((Segment)element).details.size()  != 0;
		}
	
		/*
		 * @see ITreeContentProvider#getParent(Object)
		 */
		public Object getParent(Object element) {
			if (element instanceof Segment)
				return ((Segment)element).parent;
			return null;
		}

		/*
		 * @see ITreeContentProvider#getChildren(Object)
		 */
		public Object[] getChildren(Object element) {
			if (element instanceof Segment)
				return ((Segment)element).details.toArray();
			return new Object[0];
		}

		public void setContent(List fContent) {
			this.fContent = fContent;
		}

		public List getContent() {
			return fContent;
		}
	}
/**
 * A content outline page which always represents the content of the
 * connected editor in 10 segments.
 */
public class SableccContentOutlinePage extends ContentOutlinePage {
	protected final static String PRODUCTIONS = "__sablecc_PRODUCTIONS";
	protected final static String IGNRED_TOKENS =
		"__sablecc_IGNORED_TOKENS";
	protected final static String TOKENS = "__sablecc_TOKENS";
	/*
	 * fields modified and added:
	 * 1. segment string for 
	 * 			helper definition
	 * 			tokens
	 * 			ignored tokens
	 * 			productions
	 * 2. fContent is a list of lists.
	 */
	
	protected final static String SEGMENTS = "__sablecc_segments"; //$NON-NLS-1$


	public final static String HELPERS= "__sablecc_HELPERS";
	protected Object fInput;
	protected IDocumentProvider fDocumentProvider;

	protected ITextEditor fTextEditor;

	/**
	 * Creates a content outline page using the given provider and the given editor.
	 */
	public SableccContentOutlinePage(
		IDocumentProvider provider,
		ITextEditor editor) {
		super();
		fDocumentProvider = provider;
		fTextEditor = editor;
	}

	/* (non-Javadoc)
	 * Method declared on ContentOutlinePage
	 */
	public void createControl(Composite parent) {
		/**
		 * several things are done here:
		 * 1 get the viewer inside the content outline page.
		 * 2 set contentprovider and labelprovider for this viewer
		 * 3 set selection listener for this viewer.
		 * 4 set input for this viewer --> it will call contentprovider's input changed.
		 * 
		 */
		super.createControl(parent);

		TreeViewer viewer = getTreeViewer();
		viewer.setContentProvider(new ContentProvider(this));
		viewer.setLabelProvider(new LabelProvider());
		viewer.addSelectionChangedListener(this);

		if (fInput != null)
			viewer.setInput(fInput);
	}
	
	

	/* (non-Javadoc)
	 * Method declared on ContentOutlinePage
	 */
	public void selectionChanged(SelectionChangedEvent event) {

		super.selectionChanged(event);

		ISelection selection = event.getSelection();
		if (selection.isEmpty())
			fTextEditor.resetHighlightRange();
		else {
			Segment segment =
				(Segment) ((IStructuredSelection) selection).getFirstElement();
			int start = segment.position.getOffset();
			int length = segment.position.getLength();
			try {
				fTextEditor.setHighlightRange(start, length, true);
			} catch (IllegalArgumentException x) {
				fTextEditor.resetHighlightRange();
			}
		}
	}

	/**
	 * Sets the input of the outline page
	 */
	public void setInput(Object input) {
		fInput = input;
		update();
	}

	/**
	 * Updates the outline page.
	 */
	public void update() {
		TreeViewer viewer = getTreeViewer();

		if (viewer != null) {
			Control control = viewer.getControl();
			if (control != null && !control.isDisposed()) {
				control.setRedraw(false);
				viewer.setInput(fInput);
				viewer.expandAll();
				control.setRedraw(true);
			}
		}
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.contentoutline.ContentOutlinePage#getTreeViewer()
	 */
	public TreeViewer getTreeViewer() {
		
		return super.getTreeViewer();
	}

}


