/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package dk.eclipse.sablecceditor;


import org.eclipse.jface.text.rules.RuleBasedScanner;
import dk.eclipse.sablecceditor.sablecc.SableccCodeScanner;
import dk.eclipse.sablecceditor.sableccdoc.SableccDocScanner;
import dk.eclipse.sablecceditor.util.SableccColorProvider;

/** The JavaEditorEnvironment maintains singletons used by the java editor
 * examples.
 */
public class SableccEditorEnvironment {

	private static SableccColorProvider fgColorProvider;
	private static SableccCodeScanner fgCodeScanner;
	private static SableccDocScanner fgDocScanner;

	private static int fgRefCount= 0;

	/**
	 * A connection has occured - initialize the receiver if it is the first activation.
	 */
	public static void connect(Object client) {
		if (++fgRefCount == 1) {
			fgColorProvider= new SableccColorProvider();
			fgCodeScanner= new SableccCodeScanner(fgColorProvider);
			fgDocScanner= new SableccDocScanner(fgColorProvider);
		}
	}
	
	/**
	 * A disconnection has occured - clear the receiver if it is the last deactivation.
	 */
	 public static void disconnect(Object client) {
		if (--fgRefCount == 0) {
			fgCodeScanner= null;
			fgDocScanner= null;
			fgColorProvider.dispose();
			fgColorProvider= null;
		}
	}
	
	/**
	 * Returns the singleton scanner.
	 */
	 public static RuleBasedScanner getSableccCodeScanner() {
		return fgCodeScanner;
	}
	
	/**
	 * Returns the singleton color provider.
	 */
	 public static SableccColorProvider getSableccColorProvider() {
		return fgColorProvider;
	}
	
	/**
	 * Returns the singleton document scanner.
	 */
	 public static RuleBasedScanner getSableccDocScanner() {
		return fgDocScanner;
	}
}
