/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package dk.eclipse.sablecceditor;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.*;
import org.eclipse.jface.text.source.*;
import org.eclipse.swt.graphics.RGB;
import dk.eclipse.sablecceditor.sablecc.*;
import dk.eclipse.sablecceditor.sableccdoc.JavaDocCompletionProcessor;
import dk.eclipse.sablecceditor.util.SableccColorProvider;

/**
 * Example configuration for an <code>SourceViewer</code> which shows Java code.
 */
public class SableccSourceViewerConfiguration
	extends SourceViewerConfiguration {

	/**
	 * Single token scanner.
	 */
	static class SingleTokenScanner extends BufferedRuleBasedScanner {
		public SingleTokenScanner(TextAttribute attribute) {
			setDefaultReturnToken(new Token(attribute));
		}
	};

	/**
	 * Default constructor.
	 */
	public SableccSourceViewerConfiguration() {
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	
	public IAnnotationHover getAnnotationHover(ISourceViewer sourceViewer) {

//		return new SableccAnnotationHover();
		return null;
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	//
	public IAutoIndentStrategy getAutoIndentStrategy(
		ISourceViewer sourceViewer,
		String contentType) {
		return (
			IDocument.DEFAULT_CONTENT_TYPE.equals(contentType)
				? new SableccAutoIndentStrategy()
				: new DefaultAutoIndentStrategy());
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] {
			IDocument.DEFAULT_CONTENT_TYPE,
			SableccPartitionScanner.SABLECC_DOC,
			SableccPartitionScanner.SABLECC_MULTILINE_COMMENT };
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	 //TODO: change!
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {

		ContentAssistant assistant = new ContentAssistant();
		assistant.setContentAssistProcessor(
			new SableccCompletionProcessor(),
			IDocument.DEFAULT_CONTENT_TYPE);
		assistant.setContentAssistProcessor(
			new JavaDocCompletionProcessor(),
			SableccPartitionScanner.SABLECC_DOC);

		assistant.enableAutoActivation(true);
		assistant.setAutoActivationDelay(500);
		assistant.setProposalPopupOrientation(
			IContentAssistant.PROPOSAL_OVERLAY);
		assistant.setContextInformationPopupOrientation(
			IContentAssistant.CONTEXT_INFO_ABOVE);
		assistant.setContextInformationPopupBackground(
			SableccEditorEnvironment.getSableccColorProvider().getColor(
				new RGB(150, 150, 0)));

		return assistant;
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public String getDefaultPrefix(
		ISourceViewer sourceViewer,
		String contentType) {
		return (IDocument.DEFAULT_CONTENT_TYPE.equals(contentType) ? "//" : null); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public ITextDoubleClickStrategy getDoubleClickStrategy(
		ISourceViewer sourceViewer,
		String contentType) {
		return new SableccDoubleClickSelector();
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public String[] getIndentPrefixes(
		ISourceViewer sourceViewer,
		String contentType) {
		return new String[] { "\t", "    " }; //$NON-NLS-1$ //$NON-NLS-2$
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {

		SableccColorProvider provider =
			SableccEditorEnvironment.getSableccColorProvider();
		PresentationReconciler reconciler = new PresentationReconciler();

		DefaultDamagerRepairer dr =
			new DefaultDamagerRepairer(
				SableccEditorEnvironment.getSableccCodeScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		dr =
			new DefaultDamagerRepairer(
				new SingleTokenScanner(
					new TextAttribute(
						provider.getColor(
							SableccColorProvider.JAVADOC_DEFAULT))));
		reconciler.setDamager(dr, SableccPartitionScanner.SABLECC_DOC);
		reconciler.setRepairer(dr, SableccPartitionScanner.SABLECC_DOC);

		dr =
			new DefaultDamagerRepairer(
				new SingleTokenScanner(
					new TextAttribute(
						provider.getColor(
							SableccColorProvider.MULTI_LINE_COMMENT))));
		reconciler.setDamager(
			dr,
			SableccPartitionScanner.SABLECC_MULTILINE_COMMENT);
		reconciler.setRepairer(
			dr,
			SableccPartitionScanner.SABLECC_MULTILINE_COMMENT);

		return reconciler;
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	public int getTabWidth(ISourceViewer sourceViewer) {
		return 4;
	}

	/* (non-Javadoc)
	 * Method declared on SourceViewerConfiguration
	 */
	
	public ITextHover getTextHover(
		ISourceViewer sourceViewer,
		String contentType) {
//		return new SableccTextHover();
		return null;
	}
}
