/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package dk.eclipse.sablecceditor.sablecc;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;

import dk.eclipse.sablecceditor.util.SableccColorProvider;
import dk.eclipse.sablecceditor.util.SableccWhitespaceDetector;
import dk.eclipse.sablecceditor.util.SableccWordDetector;
/**
 * A Java code scanner.
 */
public class SableccCodeScanner extends RuleBasedScanner {
	/**
	 * @author Administrator
	 *
	 * To change the template for this generated type comment go to
	 * Window>Preferences>Java>Code Generation>Code and Comments
	 */

	private static String[] fgKeywords = { "Package", "Helpers", "States", "Ignored", "Tokens", "Productions" }; //$NON-NLS-36$ //$NON-NLS-35$ //$NON-NLS-34$ //$NON-NLS-33$ //$NON-NLS-32$ //$NON-NLS-31$ //$NON-NLS-30$ //$NON-NLS-29$ //$NON-NLS-28$ //$NON-NLS-27$ //$NON-NLS-26$ //$NON-NLS-25$ //$NON-NLS-24$ //$NON-NLS-23$ //$NON-NLS-22$ //$NON-NLS-21$ //$NON-NLS-20$ //$NON-NLS-19$ //$NON-NLS-18$ //$NON-NLS-17$ //$NON-NLS-16$ //$NON-NLS-15$ //$NON-NLS-14$ //$NON-NLS-13$ //$NON-NLS-12$ //$NON-NLS-11$ //$NON-NLS-10$ //$NON-NLS-9$ //$NON-NLS-8$ //$NON-NLS-7$ //$NON-NLS-6$ //$NON-NLS-5$ //$NON-NLS-4$ //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$

	//private static String[] fgTypes= { "void", "boolean", "char", "byte", "short", "int", "long", "float", "double" }; //$NON-NLS-1$ //$NON-NLS-5$ //$NON-NLS-7$ //$NON-NLS-6$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-2$

	//private static String[] fgConstants= { "false", "null", "true" }; //$NON-NLS-3$ //$NON-NLS-2$ //$NON-NLS-1$

	/**
	 * Creates a Java code scanner
	 */
	public SableccCodeScanner(SableccColorProvider provider) {
		/** 
		 *  we distinguish several types of tokens here.
		 *  1. keyword (red)
		 *  2. comment (grayish yellow)
		 *  3. string  (blue)
		 *  4. whitespace
		 *  //5. all symbol definitions ( but default color is green)
		 *  6. others. (black) 
		 */
		//

		IToken keyword =
			new Token(
				new TextAttribute(
					provider.getColor(SableccColorProvider.KEYWORD)));
		IToken comment =
			new Token(
				new TextAttribute(
					provider.getColor(
						SableccColorProvider.SINGLE_LINE_COMMENT)));
		IToken string =
			new Token(
				new TextAttribute(
					provider.getColor(SableccColorProvider.STRING)));

		TextAttribute symbolTextAttr= new TextAttribute(
		provider.getColor(SableccColorProvider.SYMBOL));
		
		IToken symbol = new Token(	symbolTextAttr		);
			

		IToken other =
			new Token(
				new TextAttribute(
					provider.getColor(SableccColorProvider.DEFAULT)));

		List rules = new ArrayList();

		// Add rule for single line comments.
		rules.add(new EndOfLineRule("//", comment)); //$NON-NLS-1$


		// Add generic whitespace rule.
		rules.add(new WhitespaceRule(new SableccWhitespaceDetector()));

		// Add word rule for keywords, types, and constants.
		WordRule wordRule = new WordRule(new SableccWordDetector(), Token.UNDEFINED);
		for (int i = 0; i < fgKeywords.length; i++)
			wordRule.addWord(fgKeywords[i], keyword);
		rules.add(wordRule);
		
		// Add rule for strings and character constants.
		//rules.add(new SingleLineRule("\"", "\"", string, '\\')); //$NON-NLS-2$ //$NON-NLS-1$
		//rules.add(new SingleLineRule("'", "'", string, '\\')); //$NON-NLS-2$ //$NON-NLS-1$
	    
		SableccStrLitRule strRule= new SableccStrLitRule(Token.UNDEFINED);
		strRule.setToken(string);
		rules.add(strRule);
		
		
		// Add symbol definition rule for symbol definitions.
		SableccSymDefRule symRule= new SableccSymDefRule(new SableccWordDetector(),other);
		symRule.setToken(symbol);
		rules.add(symRule);

		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
		
	}
}

 class SableccStrLitRule  implements IRule{
	/** Internal setting for the uninitialized column constraint */
	protected static final int UNDEFINED= -1;
	/** The table of predefined words and token for this rule */
	
	private StringBuffer fBuffer = new StringBuffer();
	/** The column constraint */
	protected int fColumn= UNDEFINED;
	/** The default token to be returned on success and if nothing else has been specified. */
	protected IToken fDefaultToken;
	
	/** The word detector used by this rule */
	protected IWordDetector fDetector;
	private IToken fToken;
	/**
	 * @param detector
	 */
	public SableccStrLitRule() {
		
	}
	
	/**
	 * @param detector
	 * @param defaultToken
	 */
	public SableccStrLitRule(IToken defaultToken) {
		fDefaultToken= defaultToken;
		// 
	}
	
	public IToken evaluate(ICharacterScanner scanner) {
		int c = scanner.read();
		if ('\''==(char)c) {
			/* 2 cases
			 *  1. it's a '''.
			 *  2. it's a 'xxxxx', where x can't be ' and linefeed.
			 */
			// case 1.
			int n;
			n= scanner.read();
			if (n== '\'') {
				n= scanner.read();
				if (n== '\'')
					return fToken;
				else {
					scanner.unread();
					scanner.unread();
				}
			}else scanner.unread();
				
			
			//case 2.
			
			if (fColumn == UNDEFINED | (fColumn == scanner.getColumn() - 1)) {
				fBuffer.setLength(0);
				
				
				do {
					fBuffer.append((char)c);
					c=scanner.read();
				}while ( c!=ICharacterScanner.EOF && 
						(char)c!='\'' && (char)c!= 13 && (char)c!= 10);
				
				if (c=='\''){
					fBuffer.append((char)c);
					return fToken;	
				}
				
				
				scanner.unread();				 
				if (fDefaultToken.isUndefined())
						unreadBuffer(scanner);
				return fDefaultToken;
			}
		}
	
		scanner.unread();
		return Token.UNDEFINED;
	}
		
	public void setToken(IToken aToken){
		fToken=aToken;
	}
	
	protected void unreadBuffer(ICharacterScanner scanner) {
		for (int i = fBuffer.length() - 1; i >= 0; i--)
			scanner.unread();
	}
	
}

 class SableccSymDefRule extends WordRule {
	private StringBuffer fBuffer = new StringBuffer();
	private IToken fToken;
	/**
	 * @param detector
	 */
	public SableccSymDefRule(IWordDetector detector) {
		super(detector);
		// 
	}
	
	/**
	 * @param detector
	 * @param defaultToken
	 */
	public SableccSymDefRule(IWordDetector detector, IToken defaultToken) {
		super(detector, defaultToken);
		// 
	}
	
	public IToken evaluate(ICharacterScanner scanner) {
		int c = scanner.read();
		if (fDetector.isWordStart((char) c)) {
			if (fColumn == UNDEFINED
				|| (fColumn == scanner.getColumn() - 1)) {
				fBuffer.setLength(0);
				do {
					fBuffer.append((char) c);
					c = scanner.read();
				} while (
					c != ICharacterScanner.EOF
						&& fDetector.isWordPart((char) c));
				scanner.unread();
	
				/* now assume we can continue to read.
				 * until a non-space character is met. 
				 * check if it's a '='.
				 */
				int back = 0;
				do {
					c = scanner.read();
					back++;
				} while (Character.isWhitespace((char) c));
	
				// if eof, then assume it is not a symbol definition.
				if (c != '=') {
					for (int i = back; i > 0; i--)
						scanner.unread();
					if (fDefaultToken.isUndefined())
						unreadBuffer(scanner);
					return fDefaultToken;
				}
	
				return fToken;
			}
		}
	
		scanner.unread();
		return Token.UNDEFINED;
	}
		
	public void setToken(IToken aToken){
		fToken=aToken;
	}
	
	protected void unreadBuffer(ICharacterScanner scanner) {
		for (int i = fBuffer.length() - 1; i >= 0; i--)
			scanner.unread();
	}
	
}